// file: $DT/class/tree/v1.0/dt_bayes_1.cc
//

// isip include files
//
#include "attribute_node.h"
#include "tree_node.h"
#include "tree.h"

#include <integral_constants.h>
#include "attribute_node_constants.h"
#include "tree_node_constants.h"
#include "decision_tree.h"

// method: continuous_bayes_split_cc
//
// arguments: 
//  float_4& max_split_a: (input) the maximum mask
//  Attribute_node* attr_node_a: (input) the attribute node we are
//                                     interested in
//  Tree_node* node_a: (input) the tree node we are currently trying to split
//
// return: logical_1 indicating success
//
// this method 
//
float_4 Decision_tree::continuous_bayes_split_cc(float_4& max_split_a,
						 Attribute_node* attr_node_a,
						 Tree_node* node_a) {

  // get the linked list containing the data nodes for this tree node
  //
  Linked_list* data_list;
  if (node_a->get_data_nodes_cc(data_list) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to get data nodes from tree node");
  }

  // get the number of data nodes in the linked list
  //
  int_4 num_data_nodes = 0;
  if (node_a->get_num_data_nodes_cc(num_data_nodes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to get number of data nodes in tree node");
  }
  
  if (num_data_nodes <= 0) {
    error_handler_cc((char_1*)"continous_bayes_split",
		     (char_1*)"num_data_nodes <= 0");
  }

  // get the attribute label
  //
  char_1* attr_label = new char_1 [ISIP_MAX_STRING_LENGTH];
  if (attr_node_a->get_label_cc(attr_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split",
		     (char_1*)"unable to get attribute label");
  }
  
  float_4* attr_values = new float_4 [num_data_nodes];

  // loop over all data nodes
  //
  for (int_4 i=0; i<num_data_nodes; i++) {
    Data_node* data_node = (Data_node*)data_list->get_node_cc();
    if (data_node == (Data_node*)NULL) {
      error_handler_cc((char_1*)"continous_bayes_split",
		       (char_1*)"NULL data node");
    }

    // get the attribute values
    //
    if (data_node->get_value_cc(attr_values[i], attr_label) != ISIP_TRUE) {
      error_handler_cc((char_1*)"continous_bayes_split",
		       (char_1*)"unable to get attribute value");
    }
    data_list = data_list->get_next_cc();
  }
  
  // sort the attribute values
  //
  int_4* index = new int_4 [num_data_nodes];
  if (qsort_cc(index, attr_values, num_data_nodes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to sort data");
  }

  // loop through all possible splits for this attribute
  //
  char_1* class_label = new char_1[ISIP_MAX_STRING_LENGTH];
  int_4 num_classes;
  float_4 max_prob = DT_MINIMUM_FLOAT_4;

  // get the class label
  //
  if (class_node_d->get_label_cc(class_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continuous_bayes_split_cc",
                     (char_1*)"unable to get class label from class node");
  }

  // get the number of classes
  //
  if (class_node_d->get_num_values_cc(num_classes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
                     (char_1*)"unable to get the number of classes");
  }

  for (int_4 i=0; i<num_data_nodes-1; i++) {
    if (attr_values[index[i]] != attr_values[index[i+1]] ) {
      float_4 split_value = (attr_values[index[i]]+attr_values[index[i+1]])/2;
      float_4 left_prob = calculate_bayes_left_prob_cc(split_value,
					       class_label, num_classes,
					       attr_label, node_a);
      float_4 right_prob = calculate_bayes_right_prob_cc(split_value,
						 class_label, num_classes,
						 attr_label, node_a);

      // keep the maximum split
      //
      float_4 total_prob = left_prob + right_prob;
      
      if (total_prob > max_prob) {
	max_prob = total_prob;
	max_split_a = split_value;
      }
    }
  }
  
  // clean up
  //
  if (attr_label != (char_1*)NULL) {
    delete [] attr_label;
    attr_label = (char_1*)NULL;
  }

  if (attr_values != (float_4*)NULL) {
    delete [] attr_values;
    attr_values = (float_4*)NULL;
  }

  if (index != (int_4*)NULL) {
    delete [] index;
    index = (int_4*)NULL;
  }

  if (class_label != (char_1*)NULL) {
    delete [] class_label;
    class_label = (char_1*)NULL;
  }
  
  // return maximum value 
  //
  return max_prob;
}
