// file: $DT/class/tree/v1.0/dt_bayes_3.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "attribute_node.h"
#include "tree_node.h"
#include "tree.h"

#include <integral_constants.h>
#include "tree_node_constants.h"
#include "attribute_node_constants.h"
#include "decision_tree.h"

// method: calculate_bayes_right_prob_cc
//
// arguments:
//  int_4* mask_a: (input) indicate which attribute goes to the right
//                 and which goes to the right
//  char_1* class_label_a: (input)
//  int_4 num_classes_a: (input) the number of classes
//  int_4 num_categories_a: (input) number of categories for this attribute
//  Attribute_node* attr_node_a: (input) the attribute node we are interestedin
//  Tree_node* node_a: (input) the tree node we are currently trying to split
//
// return: logical_1 indicating success
//
// this method calculates the right probability of the split indicated
// by mask_a
//
//
float_4 Decision_tree::calculate_bayes_right_prob_cc (int_4* mask_a,
						     char_1* class_label_a,
						     int_4 num_classes_a,
						     char_1* attr_label_a,
						     int_4 num_attrs_a,
						     char_1** attr_values_a,
						     Tree_node* node_a) {

  // get the linked list containing the data nodes for this tree node
  //
  Linked_list* data_list;
  if (node_a->get_data_nodes_cc(data_list) != ISIP_TRUE) {
    error_handler_cc((char_1*)"calculate_bayes_right_prob_cc",
		     (char_1*)"unable to get data nodes at this tree node");
  }

  // loop over all the data nodes
  //
  char_1* class_value = new char_1 [ISIP_MAX_STRING_LENGTH];
  int_4* class_count = new int_4[num_classes_a];
  memset (class_count, 0, sizeof(int_4) * num_classes_a);
  char_1* attr_value = new char_1 [ISIP_MAX_STRING_LENGTH];
  int_4 right_count = 0;

  while (data_list != (Linked_list*)NULL) {
    Data_node* data_node = (Data_node*)data_list->get_node_cc();

    // get the class value for this node
    //
    if (data_node->get_item_cc(class_value, class_label_a) != ISIP_TRUE) {
      error_handler_cc((char_1*)"continous_bayes_split_cc",
		       (char_1*)"unable to get class value");
    }

    // get the attribute value of this attribute label for this node
    //
    if (data_node->get_item_cc(attr_value, attr_label_a) != ISIP_TRUE) {
      error_handler_cc((char_1*)"calculate_bayes_right_prob_cc",
		       (char_1*)"unable get attribute value");
    }

    for (int_4 i=0; i<num_attrs_a; i++) {
      if ((strcmp((char*)attr_value, (char*)attr_values_a[i]) == 0) &&
	  (mask_a[i] == DT_SPLIT_RIGHT)) {

	right_count++;
	
	for (int_4 j=0; j<num_classes_a; j++) {
	  if (strcmp((char*)class_value, (char*)class_names_d[j]) == 0) {
	    class_count[j]++;
	  }
	}
      }
    }
    data_list = data_list->get_next_cc();
  }
  
  // calculate the right probability
  //
  float_4 right_prob = 0;

  // check to avoid division by zero
  //
  if (right_count > 0) {
    for (int_4 i=0; i<num_classes_a; i++) {
      if (class_count[i] > 0) {
	right_prob += class_count[i] * log10((float_4)class_count[i]/right_count);
      }
    }
  }
  
  // clean up
  //
  if (class_value != (char_1*)NULL) {
    delete [] class_value;
    class_value = (char_1*)NULL;
  }

  if (class_count != (int_4*)NULL) {
    delete [] class_count;
    class_count = (int_4*)NULL;
  }

  if (attr_value != (char_1*)NULL) {
    delete [] attr_value;
    attr_value = (char_1*)NULL;
  }
  
  // return maximum value 
  //
  return right_prob;
}

