// file: $DT/class/tree/v1.0/dt_cal_4.cc
//
// isip include files
//
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "linked_list.h"
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>

// method: calculate_vector_entropy_cc
//
// arguments:
//  int_4** mask_vector (output): the list of best masks used for each
//                                attribute
//  float_4* cont_vector (output): the list of splitting values
//  float_4* calc_vector (output): the calculation results
//  Tree_node* node (input): the node where the splitting occurs
//
// return: logical_1 indicating success
//
// this method calculates the gain ratio for splitting at each attribute
// and returns a vector with calculated results along with the masks
// and the splitting values
//
logical_1 Decision_tree::calculate_vector_gainratio_cc(int_4** mask_vector_a,
						       float_4* cont_vector_a,
						       float_4* calc_vector_a,
						       Tree_node* node_a) {
  
  Linked_list* ll_a = attribute_list_d;
  Attribute_node* attr_node_a;
  int_4 type_a;
  int_4 num_attr_value_a;
  
  for (int_4 i=0;i<num_of_attributes_d;i++) {

    attr_node_a = (Attribute_node*) ll_a->get_node_cc();

    // determine if attribute is discrete or continuous
    //
    attr_node_a->get_type_cc(type_a);

    // calculate the vector for each attribute
    //
    if (type_a == ATTR_TYPE_DISCRETE) {

      // generate mask for discrete attribute
      //
      attr_node_a->get_num_values_cc(num_attr_value_a);
      cont_vector_a[i] = (float_4)NULL;
      mask_vector_a[i] = new int_4[num_attr_value_a];
      calc_vector_a[i] = discrete_gainratio_split_cc (mask_vector_a[i],
						      num_attr_value_a,
						      attr_node_a, node_a);
    }

    // continuous attribute
    //
    else {
      mask_vector_a[i] = (int_4*)NULL;
      calc_vector_a[i] = continuous_gainratio_split_cc (cont_vector_a[i],
							attr_node_a, node_a);
    }

    ll_a=ll_a->get_next_cc();
  }

  // exit gracefully; note that memory declared here will be cleaned up
  // after the value comparisons in the preceeding method
  //
  return ISIP_TRUE;

}
