// file: $DT/class/tree/v1.0/dt_cal_6.cc
//
// isip include files
//
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "tree_node.h"
#include "tree.h"
#include "linked_list.h"
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>

// method: calculate_pessimistic_prune_cc
//
// arguments:
//  Tree_node* node (input): the node where the pruning is considered
//  Tree_node* left (input): left child of the node
//  Tree_node* right (input): right child of the node
//
// return: logical_1 indicating if pruning is needed
//
// this method calculate the predicted errors of the subtree and the
// pruned tree to decide if the subtree needs to be pruned
//
logical_1 Decision_tree::calculate_pessimistic_prune_cc(Tree_node* node_a,
							Tree_node* left_a,
							Tree_node* right_a,
							int_4& max_index) {

  float_4 prune_value = 0;
  int_4 data_nodes = 0;
  int_4 num_classes;
  
  node_a->get_num_classes_cc (num_classes);
  int_4* count_left = new int_4[num_classes];
  left_a->get_count_cc(count_left);
  int_4* count_right = new int_4[num_classes];
  right_a->get_count_cc(count_right);
  int_4* total_count = new int_4[num_classes];

  // find the best possible class to be pruned to
  //
  max_index = 0;
  int_4 max_count = 0;
  for (int_4 i=0;i<num_classes;i++) {

    total_count[i] = count_left[i] + count_right[i];
    data_nodes += total_count[i];

    if (total_count[i] > max_count) {
      max_count = total_count[i];
      max_index = i;
    }
  }    

  delete [] count_left;
  delete [] count_right;

  prune_value = data_nodes *
    confidence_limit_cc(data_nodes, data_nodes - total_count[max_index]);

    
  float_4 left_prune = left_a->get_prune_value_cc();
  float_4 right_prune = right_a->get_prune_value_cc();
  float_4 subtree_value = left_prune + right_prune;

  if (subtree_value >= prune_value) {
    node_a->set_prune_value_cc(prune_value);
    return ISIP_TRUE;
  }
  else {
    node_a->set_prune_value_cc(subtree_value);
  }
  
  // exit gracefully
  //
  return ISIP_FALSE;

}
