// file: $DT/class/decision_tree/v1.0/dt_set_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>

// method: set_param_cc
//
// arguments:
//  char_1* param: (input) parameter string
//  char_1* value: (input) value string
//
// return: a logical_1 indicating status
//
// This function sets a given parameter (param) to a value (value)
//
logical_1 Decision_tree::set_param_cc (char_1* param_a, char_1* value_a) {

  // check for valid arguments
  //
  if (param_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"set_param_cc",
		     (char_1*)"invalid parameter pointer");
  }
  
  if (value_a == (char_1*)NULL) {
    error_handler_cc((char_1*)"set_param_cc",
		     (char_1*)"invalid value to set");
  }

  // set train, prune, or test mode
  //
  if (strcmp ((char*) param_a, (char*)DT_MODE) == 0) {
    if (strcmp ((char*) value_a, (char*)DT_MODE_TEST_NAME) == 0) {
      mode_d = DT_MODE_TEST;
    }
    else if (strcmp ((char*) value_a, (char*)DT_MODE_TRAIN_NAME) == 0) {
      mode_d = DT_MODE_TRAIN;
    }
    else if (strcmp ((char*) value_a, (char*)DT_MODE_PRUNE_NAME) == 0) {
      mode_d = DT_MODE_PRUNE;
    }
    else {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid value for mode parameter");
    }
  }

  // set splitting rule
  //
  else if (strcmp ((char*) param_a, (char*)DT_SPLIT) == 0) {
    if (strcmp ((char*) value_a, (char*)DT_SPLIT_ENT_NAME) == 0) {
      mode_split_d = DT_SPLIT_ENT;
    }
    else if (strcmp ((char*) value_a, (char*)DT_SPLIT_BAYES_NAME) == 0) {
      mode_split_d = DT_SPLIT_BAYES;
    }
    else if (strcmp ((char*) value_a, (char*)DT_SPLIT_GAINRATIO_NAME) == 0) {
      mode_split_d = DT_SPLIT_GAINRATIO;
    }
    else if (strcmp ((char*) value_a,
		     (char*)DT_SPLIT_IND_GAINRATIO_NAME) == 0) {
      mode_split_d = DT_SPLIT_IND_GAINRATIO;
    }
    else {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid splitting parameter");
    }
  }
  
  // set splitting search method
  //
  else if (strcmp ((char*) param_a, (char*)DT_SEARCH) == 0) {
    if (strcmp ((char*) value_a, (char*)DT_SEARCH_COMPLETE_NAME) == 0) {
      mode_search_d = DT_SEARCH_COMPLETE;
    }
    else if (strcmp ((char*) value_a, (char*)DT_SEARCH_FAST_NAME) == 0) {
      mode_search_d = DT_SEARCH_FAST;
    }
    else {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid splitting search method parameter");
    }
  }
  
  // set pruning rule
  //
  else if (strcmp ((char*) param_a, (char*)DT_PRUNE) == 0) {
    if (strcmp ((char*) value_a, (char*)DT_PRUNE_PESSIMISTIC_NAME) == 0) {
      mode_prune_d = DT_PRUNE_PESSIMISTIC;
    }
    else if (strcmp ((char*) value_a,
		     (char*)DT_PRUNE_COSTCOMPLEXITY_NAME) == 0) {
      mode_prune_d = DT_PRUNE_COSTCOMPLEXITY;
    }
    else {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid pruning parameter");
    }
  }
  
  // set smoothing rule
  //
  else if (strcmp ((char*) param_a, (char*)DT_SMOOTH) == 0) {
    if (strcmp ((char*) value_a, (char*)DT_SMOOTH_BAYES_NAME) == 0) {
      mode_smooth_d = DT_SMOOTH_BAYES;
    }
    else {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid smoothing parameter");
    }
  }
  
  // set display mode
  //
  else if (strcmp ((char*) param_a, (char*)DT_DISPLAY) == 0) {
    if (strcmp ((char*) value_a, (char*)DT_DISPLAY_BASIC_NAME) == 0) {
      mode_display_d = DT_DISPLAY_BASIC;
    }
    else if (strcmp ((char*) value_a, (char*)DT_DISPLAY_COUNT_NAME) == 0) {
      mode_display_d = DT_DISPLAY_COUNT;
    }
    else if (strcmp ((char*) value_a, (char*)DT_DISPLAY_PROB_NAME) == 0) {
      mode_display_d = DT_DISPLAY_PROB;
    }
    else {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid display parameter");
    }
  }

  // set tree file name
  //
  else if (strcmp ((char*) param_a, (char*)DT_TREE_FILENAME) == 0) {
    if (tree_fname_d == (char_1*)NULL) {
      tree_fname_d = new char_1[strlen((char*)value_a)+1];
      strcpy ((char*)tree_fname_d, (char*)value_a);
    }
    else {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"can't define the tree file more than once");
    }
  }

  // set confidence limit
  //
  else if (strcmp ((char*) param_a, (char*)DT_CONFIDENCE_LIMIT) == 0) {
    if (convert_string_cc(confidence_limit_d, value_a) != ISIP_TRUE) {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid confidence limit to set");
    }
    else if ((confidence_limit_d < 0) || (confidence_limit_d > 1)) {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid value for confidence limit");
    }
  }

  // set prior model
  //
  else if (strcmp ((char*) param_a, (char*)DT_PRIOR_MODEL) == 0) {
    if (strcmp ((char*) value_a, (char*)DT_PRIOR_MODEL_UNIFORM_NAME) == 0) {
      prior_model_d = DT_PRIOR_MODEL_UNIFORM;
    }
    else {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid prior model parameter");
    }
  }
  
  // set dirichlet prior
  //
  else if (strcmp ((char*) param_a, (char*)DT_DIRICHLET_PRIOR) == 0) {
    if (convert_string_cc(dirichlet_prior_d, value_a) != ISIP_TRUE) {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid dirichlet prior to set");
    }
    else if (dirichlet_prior_d < 0) {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid value for dirichlet prior");
    }
  }
  
  // set debug level
  //
  else if (strcmp ((char*) param_a, (char*)DT_DEBUG_LEVEL) == 0) {
    int_4 debug_level_a;
    if (convert_string_cc(debug_level_a, value_a) != ISIP_TRUE) {
      error_handler_cc((char_1*)"set_param_cc",
		       (char_1*)"invalid debug level to set");
    }
    else {
      set_debug_cc(debug_level_a);
    }
  }

  // invalid setting argument
  //
  else {
    error_handler_cc((char_1*)"set_param_cc",
		     (char_1*)"invalid parameter to set");
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}


