// file: $DT/class/tree/v1.0/dt_split_10.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "tree.h"    
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>

// method: calculate_continuous_split_cc
//
// arguments: attr_node:  the attribute is being split on
//            node: node for which the split is being found
//
// return: logical_1 indicating success
//
// this method determines the best discrete for a node and then
// applies this split
//
logical_1 Decision_tree::apply_best_split_cc(Attribute_node* attr_a,
					     int_4* mask_a,
					     float_4 cont_split_a,
					     Tree_node* node_a,
					     Tree_node* left_child,
					     Tree_node* right_child) {

  logical_1 purity;
  int_4 attr_type_a;
  int_4 num_classes_a;
  
  // retrieve attr_type_a and the number of classes
  //
  attr_a->get_type_cc(attr_type_a);
  class_node_d->get_num_values_cc(num_classes_a);
  float_4* left_prob = new float_4 [num_classes_a];
  float_4* right_prob = new float_4 [num_classes_a];
  
  // apply the split to left and right hand children nodes
  //
  if (attr_type_a == ATTR_TYPE_DISCRETE) {
    purity = purity_check_cc(mask_a, node_a, attr_a);
    if (purity == ISIP_FALSE) {
      apply_split_cc(left_prob, right_prob, mask_a, attr_a, node_a,
		     left_child, right_child);
    }
  }
  else {
    purity = purity_check_cc(cont_split_a, node_a, attr_a);
    if (purity == ISIP_FALSE) {
      apply_split_cc(left_prob, right_prob, cont_split_a, attr_a, node_a,
		     left_child, right_child);
    }
  }

  if (purity == ISIP_FALSE) {
    set_children_cc (left_child, right_child, node_a, left_prob, right_prob);
  }

  // clean up
  //
  if (left_prob != (float_4*)NULL) {
    delete [] left_prob;
  }
  
  if (right_prob != (float_4*)NULL) {
    delete [] right_prob;
  }
					     
  // exit gracefully
  //
  if (purity == ISIP_TRUE) {
    return ISIP_FALSE;
  }
  else {
    return ISIP_TRUE;
  }
}


