// file: $DT/class/tree/v1.0/dt_split_2.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "tree.h"    
#include "decision_tree.h"
#include "decision_tree_constants.h"
#include <integral_constants.h>

// method: apply_split_cc
//
// arguments: int_4* mask_a  mask for the discrete values
//            attr_node_a  attribute where the split occurs
//            parent_a  node to be split
//            left_a, right_a   children to which the split is applied
//
// return: logical_1 indicating success
//
// this method applies the already determined best split to the node
// this split is for the discrete split
// in the mask:  the zero's go left, the one's go right
//
logical_1 Decision_tree::apply_split_cc(float_4* left_prob_a,
					float_4* right_prob_a, int_4* mask_a,
					Attribute_node* attr_node_a,
					Tree_node* parent_a, Tree_node* left_a,
					Tree_node* right_a) {

  Linked_list* ll_data;
  Data_node* data_a;
  int_4 num_values_a;
  int_4 split_direction;
  char_1* attr_name_a= new char_1[ISIP_MAX_STRING_LENGTH];

  // get length of the mask
  //
  attr_node_a->get_num_values_cc(num_values_a);

  // get the linked list of data at the parent node
  //
  parent_a->get_data_nodes_cc(ll_data);

  // get some info about the class
  //
  int_4 num_classes;
  class_node_d->get_num_values_cc(num_classes);

  char_1* class_label = new char_1[ISIP_MAX_STRING_LENGTH];
  class_node_d->get_label_cc(class_label);

  char_1** class_values = new char_1*[num_classes];
  for (int_4 i=0; i<num_classes; i++) {
    class_values[i] = new char_1[ISIP_MAX_STRING_LENGTH];
  }
  class_node_d->get_values_cc(class_values);
    
  //  count how many instances in each class
  //
  int_4* left_count = new int_4[num_classes];
  int_4* right_count = new int_4[num_classes];
  int_4 total_count = 0;
  
  // initialize the counters
  //
  for (int_4 i=0; i<num_classes; i++) {
    left_count[i] = 0;
    right_count[i] = 0;
  }

  // classsify each data node
  //
  char_1* this_class = new char_1[ISIP_MAX_STRING_LENGTH];
  
  while ( ll_data != (Linked_list*)NULL ) {
    
    data_a=(Data_node*)ll_data->get_node_cc();

    // determine if the data goes in the left or right node
    //
    split_direction=classify_data_node_cc(mask_a, attr_node_a, data_a);

    // add pointer to data node to correct tree node child
    //
    if ( split_direction == DT_SPLIT_LEFT ) {
      left_a->add_data_node_cc(data_a);

      // get the class that this node belong to
      //
      data_a->get_item_cc(this_class, class_label);

      // compare the class for this data node to see which class it belongs
      //
      for (int_4 i=0; i<num_classes; i++) {
	if (strcmp((char*)this_class, (char*)class_values[i]) == 0) {
	  left_count[i]++;
	  break;
	}
      }
    }
    else {
      right_a->add_data_node_cc(data_a);
      
      // get the class that this node belong to
      //
      data_a->get_item_cc(this_class, class_label);
      
      // compare the class for this data node to see which class it belongs
      //
      for (int_4 i=0; i<num_classes; i++) {
	if (strcmp((char*)this_class, (char*)class_values[i]) == 0) {
	  right_count[i]++;
	  break;
	}
      }
    }

    total_count++;
    
    // get the next data node pointer
    //
    ll_data=ll_data->get_next_cc();
  }

  // add info to the nodes
  //
  attr_node_a->get_label_cc( attr_name_a );
  left_a->set_node_cc( mask_a, attr_node_a, DT_SPLIT_LEFT, TREE_NODE_EQ,
		       attr_name_a);
  right_a->set_node_cc(mask_a, attr_node_a, DT_SPLIT_RIGHT, TREE_NODE_EQ,
		       attr_name_a);

  // calculate the probability
  //
  for (int_4 i=0; i<num_classes; i++) {
    left_prob_a[i] = (float)left_count[i]/(float)total_count;
    right_prob_a[i] = (float)right_count[i]/(float)total_count;
  }

  // clean up
  //
  delete [] attr_name_a;

  if (class_label != (char_1*)NULL) {
    delete [] class_label;
  }
  for (int_4 i=0; i<num_classes; i++) {
    if (class_values[i] != (char_1*)NULL) {
      delete [] class_values[i];
    }
  }

  if (class_values != (char_1**)NULL) {
    delete [] class_values;
  }

  //  count how many instances in each class
  //
  if (left_count != (int_4*)NULL) {
    delete [] left_count;
  }

  if (right_count != (int_4*)NULL) {
    delete [] right_count;
  }

  if (this_class != (char_1*)NULL) {
    delete [] this_class;
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}


