// file: $DT/class/tree/v1.0/dt_split_7.cc
//

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "decision_tree.h"
#include "decision_tree_constants.h"

// systme include files
//
#include <integral_constants.h>

// method: purity_check_cc
//
// arguments:
//  int_4* mask_a (input): split indicator for discrete values
//
// return: ISIP_TRUE if the node is splittable
//         ISIP_FALSE if the node is not splittable
//
// this method determines if the tree is trying to split into subtrees
// with one leaf has 0 cases in the training data
//
logical_1 Decision_tree::purity_check_cc (int_4* mask_a, Tree_node* node_a,
					  Attribute_node* attr_node_a) {

  Linked_list* ll_data;
  Data_node* data_a;
  int_4 split_direction;
  int_4 num_values_a;
  int_4 left=0, right=0;
  logical_1 purity;
  logical_1 same_split = ISIP_FALSE;
  logical_1 same_class = ISIP_TRUE;
  char_1* class_label = (char_1*)NULL;
  char_1* class_name_1=new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* class_name_2=(char_1*)NULL;

  // get length of the mask
  //
  attr_node_a->get_num_values_cc(num_values_a);

  // get the linked list of data at the tree node
  //
  node_a->get_data_nodes_cc(ll_data);

    // retrieve the class label
  //
  class_label = new char_1[ISIP_MAX_STRING_LENGTH];
  if (class_node_d->get_label_cc(class_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"purity_check_cc",
                     (char_1*)"unable to get class label from class node");
  }
  
  // classsify each data node
  //
  while ( ll_data != (Linked_list*)NULL ) {
 
    data_a=(Data_node*)ll_data->get_node_cc();

    // get the class of this data node
    //
    data_a->get_item_cc(class_name_1, class_label);
 
    if ( class_name_2 == (char_1*)NULL ) {
      class_name_2=new char_1[ISIP_MAX_STRING_LENGTH];
      strcpy((char*)class_name_2,(char*)class_name_1);
    }

    // if the previous classification is not the same as the present
    // classification, then the node is not pure
    //
    if ( strcmp((char*)class_name_1,(char*)class_name_2) != 0 ) {
      same_class = ISIP_FALSE;
    }

    // determine if the data goes in the left or right node
    //
    split_direction=classify_data_node_cc(mask_a, attr_node_a, data_a);
    
    if (split_direction == DT_SPLIT_LEFT) {
      left++;
    }
    else if (split_direction == DT_SPLIT_RIGHT) {
      right++;
    }
    ll_data = ll_data->get_next_cc();
  }

  // only check to see if a split happens if the classes are not pure
  //
  if ((left == 0) || (right == 0)) {
    same_split = ISIP_TRUE;
  }

  // determine purity based on split made and on class membership
  //
  if ( same_class ) {
    purity = ISIP_TRUE;
  }
  else if ( same_split ) {
    purity = ISIP_TRUE;
  }
  else {
    purity = ISIP_FALSE;
  }

  // free up memory
  //
  if (class_label != (char_1*)NULL) {
    delete [] class_label;
  }
  if ( class_name_1 != (char_1*)NULL ) {
    delete [] class_name_1;
  }
  if ( class_name_2 != (char_1*)NULL ) {
    delete [] class_name_2;
  }

  // return the purity of this node
  //
  return purity;

}

