// file: $DT/class/tree/v1.0/tree_io_2.cc
//

// isip include files
//
#include <tree.h>
#include "tree_constants.h"
#include <string.h>
#include <integral_constants.h>

// forward class definitions
//
#include <tree_node.h>

// method: output_detail_tree_cc
//
// arguments: file pointer
//            Tree_node* points to starting tree node
//    
// return: logical_1 indicating success
//
// this method outputs the contents of the tree to a file. this is the
// tree that will be read in for testing of the tree.  it SHOULD NOT be
// modified by the user.  it contains detailed info for loading the tree.
//
logical_1 Tree::output_detail_tree_cc(FILE* fp_a, Tree_node* current_a) {

  Linked_list* child_a;

  if (fp_a == (FILE*)NULL) {
    error_handler_cc((char_1*)"output_tree_cc",
		     (char_1*)"no file opened");
    return ISIP_FALSE;
  }

  // make sure tree exists
  //
  if (current_a != NULL) {

    // get the children of the node
    //
    child_a = current_a->get_children_cc();
      
    // get the tree node
    //
    if ( child_a != (Linked_list*)NULL ) {
      current_a = (Tree_node*)child_a->get_node_cc();
    }
    else {
      current_a = (Tree_node*)NULL;
    }
    
    // recursively output the question at each node
    //
    while (current_a != (Tree_node*)NULL) {
	  
      // output the contents of this node
      //
      char_1* output_str = new char_1[ISIP_MAX_STRING_LENGTH];
      current_a->get_detailed_cc(output_str);
 
      // output node contents to a file
      //
      fprintf(fp_a,"%s\n",(char*)output_str);
      delete [] output_str;

      // recursively get each child node
      //
      output_detail_tree_cc(fp_a, current_a);

      // go to the next child in the node
      //
      child_a = child_a->get_next_cc();
      
      // get the tree node
      //
      if ( child_a != (Linked_list*)NULL ) {
	current_a = (Tree_node*)child_a->get_node_cc();
      }
      else {
	current_a = (Tree_node*)NULL;
      }
    }
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
