// file: $DT/util/analyze_dtree/v1.0/analyze_dtree.cc
//

// isip include files
//
#include "analyze_dtree.h"
#include <decision_tree.h>
#include <attribute_node.h>
#include <param_file.h>
#include <linked_list.h>

#include "analyze_dtree_constants.h"
#include <integral_constants.h>

// main program
//
main(int_4 argc, char_1** argv) {

  // declare command line arguments
  //
  char_1* param_fname = (char_1*)NULL;
  char_1* attr_fname = (char_1*)NULL;
  char_1* input_fname = (char_1*)NULL;
  char_1* output_fname = (char_1*)NULL;
  
  // declare a debug parameter
  //
  int_4 debug_level = ISIP_DEBUG_NONE;
  
  // parse the command line
  //
  if (get_parameters_cc(param_fname, attr_fname, input_fname, output_fname,
			debug_level, argc, argv) == ISIP_FALSE) {
    fprintf(stdout, "%s: error in command line arguments\n", argv[0]);
    return ISIP_ERROR;
  }

  // check command line arguments before keep going
  // no sense in training the tree and realizing that we don't
  // have a file to save it under
  //
  if (param_fname == (char_1*)NULL) {
    fprintf (stdout, "no parameter file given\n");
    return ISIP_ERROR;
  }

  if (attr_fname == (char_1*)NULL) {
    fprintf (stdout, "no attribute file given\n");
    return ISIP_ERROR;
  }
  
  if (input_fname == (char_1*)NULL) {
    fprintf (stdout, "no input file given\n");
    return ISIP_ERROR;
  }

  if (output_fname == (char_1*)NULL) {
    fprintf (stdout, "no output file given\n");
    return ISIP_ERROR;
  }
  
  //  for (int_4 i=0; i<1000; i++) {
    
  // declare objects and the necessary variables
  // 
  Decision_tree* dt = new Decision_tree;

  //  fprintf(stdout, "Allocating a new decision tree: %d\n", i);
  
  // set debug level
  //
  if (dt->set_debug_cc(debug_level) != ISIP_TRUE) {
    fprintf(stdout, "unable to set debug level\n");
    return ISIP_ERROR;
  }
  
  // load in parameter file
  //
  if (dt->load_params_cc(param_fname) != ISIP_TRUE) {
    fprintf(stdout, "%s: invalid parameter file\n", param_fname);
    return ISIP_ERROR;
  }

  // load in attribute file
  //
  if (dt->load_attributes_cc(attr_fname) != ISIP_TRUE) {
    fprintf(stdout, "%s: invalid attribute file\n", attr_fname);
    return ISIP_ERROR;
  }

  // train or test
  //
  if (dt->compute_cc(input_fname, output_fname) != ISIP_TRUE) {
    fprintf(stdout, "unable to train or test\n");
    return ISIP_ERROR;
  }

  if (debug_level > ISIP_DEBUG_NONE) {
    dt->display_data_cc();
  }

  // clean up
  //
  if (dt != (Decision_tree*)NULL) {

    fprintf (stdout, "size of decision tree: %d\n", dt->size_cc());
    //    fprintf(stdout, "Deleting the decision tree: %d\n", i);
    delete dt;
    //  }
    dt = (Decision_tree*)NULL;
  }  

  if (param_fname != (char_1*)NULL) {
    delete [] param_fname;
    param_fname = (char_1*)NULL;
  }
  
  if (attr_fname != (char_1*)NULL) {
    delete [] attr_fname;
    attr_fname = (char_1*)NULL;
  }
  
  if (input_fname != (char_1*)NULL) {
    delete [] input_fname;
    input_fname = (char_1*)NULL;
  }
  
  if (output_fname != (char_1*)NULL) {
    delete [] output_fname;
    output_fname = (char_1*)NULL;
  }

  // exit gracefully
  //
  return ISIP_NO_ERROR;
}





